import os
from .mks_localization_model import MKSLocalizationModel
from .bases.primitive import PrimitiveBasis
from .bases.legendre import LegendreBasis
from .mks_structure_analysis import MKSStructureAnalysis
from .mks_homogenization_model import MKSHomogenizationModel
MKSRegressionModel = MKSLocalizationModel
DiscreteIndicatorBasis = PrimitiveBasis
ContinuousIndicatorBasis = PrimitiveBasis


def test():
    r"""
    Run all the doctests available.
    """
    import pytest
    path = os.path.split(__file__)[0]
    pytest.main(args=[path, '--doctest-modules', '-r s'])


def get_version():
    """Get the version of the code from egg_info.

    Returns:
      the package version number
    """
    from pkg_resources import get_distribution, DistributionNotFound

    try:
        version = get_distribution(__name__.split('.')[0]).version # pylint: disable=no-member
    except DistributionNotFound: # pragma: no cover
        version = "unknown, try running `python setup.py egg_info`"

    return version

__version__ = get_version()

__all__ = ['__version__',
           'test',
           'MKSLocalizationModel',
           'PrimitiveBasis',
           'LegendreBasis',
           'MKSHomogenizationModel',
           'MKSStructureAnalysis']



from .abstract import _AbstractMicrostructureBasis
try:
    import pyfftw.builders as fftmodule
except:
    import numpy.fft as fftmodule
import numpy as np


class _ImagFFTBasis(_AbstractMicrostructureBasis):
    """This class is used to make the bases that create complex valued
    microstructure functions use the standard FFT/iFFT algorithms and selects
    the appropriate fft module depending on whether or not pyfftw is installed.
    """

    def _fftn(self, X):
        """Standard FFT algorithm

        Args:
            X: NDarray (n_samples, N_x, ...)

        Returns:
            Fourier transform of X
        """
        if self._pyfftw:

            return fftmodule.fftn(np.ascontiguousarray(X),
                                  axes=self._axes, threads=self._n_jobs,
                                  planner_effort='FFTW_ESTIMATE',
                                  overwrite_input=True, avoid_copy=True)()
        else:
            return fftmodule.fftn(X, axes=self._axes)

    def _ifftn(self, X):
        """Standard iFFT algorithm

        Args:
            X: NDarray (n_samples, N_x, ...)

        Returns:
            Inverse Fourier transform of X
        """
        if self._pyfftw:
            return fftmodule.ifftn(np.ascontiguousarray(X),
                                   axes=self._axes, threads=self._n_jobs,
                                   planner_effort='FFTW_ESTIMATE',
                                   overwrite_input=True,
                                   avoid_copy=True)()
        else:
            return fftmodule.ifftn(X, axes=self._axes)



from .abstract import _AbstractMicrostructureBasis
try:
    import pyfftw.builders as fftmodule
except:
    import numpy.fft as fftmodule
import numpy as np


class _RealFFTBasis(_AbstractMicrostructureBasis):
    """This class is used to make the bases that create real valued
    microstructure functions use the real rFFT/irFFT algorithms and selects
    the appropriate fft module depending on whether or not pyfftw is installed.
    """

    def _fftn(self, X):
        """Real rFFT algorithm

        Args:
            X: NDarray (n_samples, N_x, ...)

        Returns:
            Fourier transform of X
        """
        if self._pyfftw:
            return fftmodule.rfftn(np.ascontiguousarray(X),
                                   axes=self._axes, threads=self._n_jobs,
                                   planner_effort='FFTW_ESTIMATE',
                                   overwrite_input=True, avoid_copy=True)()
        else:
            return fftmodule.rfftn(X, axes=self._axes)

    def _ifftn(self, X):
        """Real irFFT algorithm

        Args:
            X: NDarray (n_samples, N_x, ...)

        Returns:
            Inverse Fourier transform of X
        """
        if self._pyfftw:
            return fftmodule.irfftn(np.ascontiguousarray(X),
                                    s=self._axes_shape, axes=self._axes,
                                    threads=self._n_jobs,
                                    planner_effort='FFTW_ESTIMATE',
                                    avoid_copy=True)().real
        else:
            return fftmodule.irfftn(X, axes=self._axes,
                                    s=self._axes_shape).real



#!/usr/bin/env python

import subprocess
from setuptools import setup, find_packages
import os



def make_version():
    """Generates a version number using `git describe`.

    Returns:
      version number of the form "3.1.1.dev127+g413ed61".
    """
    def _minimal_ext_cmd(cmd):
        """Run a command in a subprocess.

        Args:
          cmd: list of the command

        Returns:
          output from the command
        """
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            value = os.environ.get(k)
            if value is not None:
                env[k] = value
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd,
                               stdout=subprocess.PIPE,
                               env=env).communicate()[0]
        return out

    version = 'unknown'

    if os.path.exists('.git'):
        try:
            out = _minimal_ext_cmd(['git',
                                    'describe',
                                    '--tags',
                                    '--match',
                                    'v*'])
            # ticket:475 - fix for bytecode received in Py3k
            # http://jeetworks.org/node/67
            outdecode = out.decode("utf-8")
            # convert git long-form version string, e.g.,
            # "version-3_1_1-127-g413ed61", into PEP 440 version,
            # e.g., "3.1.1.dev127+g413ed61"
            version = outdecode.strip().split("-")
            if len(version) > 1:
                version, dev, sha = version
                version = "%s.dev%s+%s" % (version[1:], dev, sha)
            else:
                version = version[0][1:]
        except OSError:
            import warnings
            warnings.warn("Could not run ``git describe``")
    elif os.path.exists('pymks.egg-info'):
        from pymks import get_version
        version = get_version()

    return version

setup(name='pymks',
      version=make_version(),
      description='Materials Knowledge Systems in Python (PyMKS)',
      author='David Brough, Daniel Wheeler',
      author_email='david.brough.0416@gmail.com',
      url='http://pymks.org',
      packages=find_packages(),
      package_data={'': ['tests/*.py']},
      )
